/*
 * Copyright (c) 2011-2014 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


/**
 * Scheduler connection related utility methods.
 */
Ext.define( 'QDesk.controller.command.connection.ConnectionUtils', {
  singleton : true,


  /**
   * Returns the item ID of the tab panel of the specified connection.
   * @param connectionId a connection ID.
   */
  getSchedulerTabItemId : function ( connectionId )
  {
    return 'schedulerTabPanel-' + connectionId;
  },


  /**
   * Returns the tab panel for the specified Quartz connection ID.
   * @param connectionId a connection ID.
   */
  getSchedulerTab : function ( connectionId )
  {
    var tabPanelItemId = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTabItemId( connectionId );
    var tabPanels = Ext.ComponentQuery.query( '#mainTabPanel > #' + tabPanelItemId );
    // assert tabPanels.size <= 1
    return Ext.isEmpty( tabPanels ) ? null : tabPanels[0];
  },


  /**
   * Returns true, if the QuartzDesk JVM Agent for the specified connection ID is available
   * and compatible with the QuartzDesk GUI.
   * @param connectionId a connection ID.
   * @return true if available and compatible, false otherwise.
   */
  isJvmAgentAvailableAndCompatible : function ( connectionId )
  {
    var jvmAgentInfo = QDesk.controller.command.connection.ConnectionUtils.getJvmAgentInfo( connectionId );
    return jvmAgentInfo.available && jvmAgentInfo.compatible;
  },


  /**
   * Returns the QuartzDesk JVM Agent Info object the specified connection ID.
   * @param connectionId a connection ID.
   * @return the info object with the following structure:
   * <pre>
   * {
     *   "available" : true,
     *   "nameAndVersion" : "QuartzDesk JVM Agent v1.3.0-SNAPSHOT",
     *   "version" : {
     *     "major" : 1,
     *     "minor" : 3,
     *     "maintenance" : 0,
     *     "qualifier" : "SNAPSHOT"
     *   },
     *   "displayVersion" : "1.3.0-SNAPSHOT"
     * }
   * </pre>
   */
  getJvmAgentInfo : function ( connectionId )
  {
    var me = this;

    var schedulerTab = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTab( connectionId );
    if ( !schedulerTab )
      Ext.Error.raise( 'Scheduler tab for connection ID: ' + connectionId + ' not found.' );

    var jvmAgentInfo = schedulerTab.getJvmAgentInfo();
    if ( !jvmAgentInfo )
      Ext.Error.raise( 'QuartzDesk JVM Agent info for connection ID: ' + me.connectionId + ' not found.' );

    return jvmAgentInfo;
  },


  /**
   * Returns the scheduler info object for the specified connection ID.
   * @param connectionId a connection ID.
   * @return the scheduler info object with the following structure:
   * <pre>
   * {
   *   "type" : "QUARTZ"
   *   "version" : {
   *     major : 1,
   *     minor : 2,
   *     maintenance : 3
   *   }
   * }
   * </pre>
   */
  getSchedulerInfo : function ( connectionId )
  {
    var me = this;

    var schedulerTab = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTab( connectionId );
    if ( !schedulerTab )
      Ext.Error.raise( 'Scheduler tab for connection ID: ' + connectionId + ' not found.' );

    var schedulerInfo = schedulerTab.getSchedulerInfo();
    if ( !schedulerInfo )
      Ext.Error.raise( 'Scheduler info for connection ID: ' + me.connectionId + ' not found.' );

    return schedulerInfo;
  },


  /**
   * Returns the node ID of the specified connection.
   * @param connectionId a connection ID.
   * @return the node ID of the specified connection.
   */
  getConnectionNodeId : function ( connectionId )
  {
    return 'connection-' + connectionId;
  },


  /**
   * Returns the name of the connection with the specified ID.
   * @param connectionId a connection ID.
   */
  getConnectionName : function ( connectionId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[0];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getConnectionNodeId( connectionId );
    var connectionNode = store.getNodeById( nodeId );
    return connectionNode.get( 'name' );
  },


  /**
   * Returns the ID of the parent folder of the specified connection, or null if not found.
   * @param connectionId a connection ID.
   * @return the parent folder ID, or null.
   */
  getConnectionParentFolderId : function ( connectionId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[0];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getConnectionNodeId( connectionId );
    var connectionNode = store.getNodeById( nodeId );
    return connectionNode ? connectionNode.parentNode.get( 'folderId' ) : null;
  },


  /**
   * Returns the node ID of the specified folder.
   * @param folderId a folder ID.
   * @return the node ID of the specified folder.
   */
  getFolderNodeId : function ( folderId )
  {
    return folderId == QDesk.util.constant.Const.CONNECTION_TREE_ROOT_FOLDER_ID ?
        QDesk.util.constant.Const.CONNECTION_TREE_ROOT_ID : 'folder-' + folderId;
  },


  /**
   * Returns the name of the folder with the specified ID.
   * @param folderId a folder ID.
   */
  getFolderName : function ( folderId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[0];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getFolderNodeId( folderId );
    var folderNode = store.getNodeById( nodeId );
    return folderNode.get( 'name' );
  },


  /**
   * Returns the ID of the parent folder of the specified folder, or null if not found.
   * @param folderId a folder ID.
   * @return the parent folder ID, or null.
   */
  getFolderParentFolderId : function ( folderId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[0];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getFolderNodeId( folderId );
    var folderNode = store.getNodeById( nodeId );
    return folderNode ? folderNode.parentNode.get( 'folderId' ) : null;
  },


  /**
   * Returns the scheduler type for the specified connection ID.
   * @param connectionId a connection ID.
   * @return the scheduler type (QDesk.enums.scheduler.SchedulerType value).
   */
  getSchedulerType : function ( connectionId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[0];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getConnectionNodeId( connectionId );
    var connectionNode = store.getNodeById( nodeId );
    return connectionNode.get( 'schedulerType' );
  }
} );
