/*
 * Copyright (c) 2011-2014 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Edits an existing connection with the specified ID in the
 * connections tree panel.
 */
Ext.define( 'QDesk.controller.command.connection.EditSchedulerConnection', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const',
    'QDesk.view.navigation.connections.EditSchedulerConnectionFormMode',
    'QDesk.view.navigation.connections.EditSchedulerConnectionWindow'
  ],

  /*
   * Required ID of the connection to edit.
   */
  connectionId : undefined,

  /**
   * Required config parameter with the scheduler type.
   * See QDesk.enums.scheduler.SchedulerType.
   */
  schedulerType : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: schedulerType' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'EditSchedulerConnectionWindow', {
      qdesk : {
        mode : QDesk.view.navigation.connections.EditSchedulerConnectionFormMode.EDIT,
        schedulerType : me.schedulerType
      }
    } );

    var formPanel = window.down( '#form' );

    window.on( 'afterrender', function ()
    {
      var nameField = formPanel.down( '#name' );
      nameField.focus( false, QDesk.util.constant.Const.FOCUS_DELAY );
    } );

    // Discover button
    formPanel.down( '#discoverButton' ).on( 'click', function ()
    {
      var me = this;
      Ext.create( 'QDesk.controller.command.connection.DiscoverSchedulers', {
        window : window,
        schedulerType : me.schedulerType
      } ).execute();
    }, me );

    // Test Connection button
    formPanel.down( '#testButton' ).on( 'click', function ()
    {
      var me = this;
      Ext.create( 'QDesk.controller.command.connection.TestSchedulerConnection', {
        window : window,
        schedulerType : me.schedulerType
      } ).execute();
    }, me );

    // Save/Update button
    formPanel.down( '#actionButton' ).on( 'click', function ()
    {
      me._submitForm( window );
    } );

    /* Load the form data */
    QDesk.util.UxUtils.maskBody( QDesk.Msg.get( 'QDesk.controller.command.connection.EditSchedulerConnection.maskLoadMsg' ) );

    formPanel.load( {
      url : QDesk.util.constant.ActionUrl.schedulerConnectionActionGetConnection,
      scope : me,
      params : {
        connectionId : me.connectionId
      },
      success : function ( form, action )
      {
        QDesk.util.UxUtils.unmaskBody();
        window.show();
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.unmaskBody();
        window.close();
        QDesk.util.UxUtils.showExtJsActionError( action );
      }
    } );
  },


  /**
   * Submits the form.
   * @param window the edit connection window.
   */
  _submitForm : function ( window )
  {
    var me = this;

    var formPanel = window.down( '#form' );

    var useServiceParams = formPanel.down( '#serviceParamsRadio' ).getValue();
    var useServiceUrl = formPanel.down( '#serviceUrlRadio' ).getValue();

    var form = formPanel.getForm();
    var fieldValues = form.getFieldValues();

    var connection = Ext.create( 'QDesk.model.Connection',
        {
          id : fieldValues.id,
          name : fieldValues.name,
          remoteJmxService : {
            protocol : useServiceParams ? fieldValues.protocol : null,
            host : useServiceParams ? fieldValues.host : null,
            port : useServiceParams ? fieldValues.port : null,
            serviceUrl : useServiceUrl ? fieldValues.serviceUrl : null,
            authentication : {
              username : fieldValues.username ? fieldValues.username : null,
              password : fieldValues.password ? fieldValues.password : null
            }
          },
          schedulerType : me.schedulerType,
          schedulerObjectName : fieldValues.schedulerObjectName
        }
    );

    if ( fieldValues.username == '' && fieldValues.password == '' )
      connection.get( 'remoteJmxService' ).authentication = null;

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.schedulerConnectionActionUpdateConnection,
      params : {
        connectionId : me.connectionId,
        connection : Ext.encode( connection.data )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.connection.EditSchedulerConnection.waitMsg',
          fieldValues.name ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          window.close();

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.connection.EditSchedulerConnection.successMsg',
                  fieldValues.name ),
              function ()
              {
                me.application.fireEvent( 'connectionUpdated', {
                  connectionId : me.connectionId
                } );
              } );

          /*
           * Close the scheduler view if it is open.
           */
          me._closeSchedulerTabForConnection();

          me._refreshTargetFolder();
        }
        else
        {
          QDesk.util.UxUtils.showJsonResponseErrors( json );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action );
      }
    } );
  },


  /**
   * Reloads the connection's parent folder.
   */
  _refreshTargetFolder : function ()
  {
    var me = this;

    // reload the connection's parent folder in the connections tree view
    var parentFolderId = QDesk.controller.command.connection.ConnectionUtils.getConnectionParentFolderId( me.connectionId );
    if ( parentFolderId )
    {
      Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
        folderId : parentFolderId
      } ).execute();
    }
    else
    {
      Ext.Error.raise( 'Parent folder ID for connection ID: ' + me.connectionId + ' not found.' );
    }
  },


  /**
   * Closes the scheduler tab (scheduler view) of the connection that is to be deleted.
   */
  _closeSchedulerTabForConnection : function ()
  {
    var me = this;

    var schedulerTabItemId = "schedulerTabPanel-" + me.connectionId;
    Ext.each( Ext.ComponentQuery.query( '#' + schedulerTabItemId ), function ( tabPanel )
    {
      tabPanel.close();
    } );
  }
} );
