/*
 * Copyright (c) 2011-2014 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Scheduler Notifications tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.notif.exec.InitializeSchedulerExecNotificationsTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.QuartzControllerUtils',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator',
    'QDesk.enums.message.MessageCreatorType',
    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleFormMode',
    'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleWindow'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var schedulerNotificationsTab = QDesk.controller.QuartzControllerUtils.getSchedulerNotificationsTab( me.connectionId );

    me._bindEventHandlers( schedulerNotificationsTab );

    schedulerNotificationsTab.setInitialized( true );
  },


  /**
   * Binds events of the specified scheduler Notifications tab to their respective handlers.
   */
  _bindEventHandlers : function ( schedulerNotificationsTab )
  {
    var me = this;
    me._bindEventHandlersNotificationRulesGrid( schedulerNotificationsTab );
  },


  /**
   * Binds the notification rules grid events to their handlers.
   */
  _bindEventHandlersNotificationRulesGrid : function ( schedulerNotificationsTab )
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.SchedulerExecutionNotifications &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId ) )
    {
      var notificationRulesGrid = schedulerNotificationsTab.down( '#execNotificationRulesGrid' );

      // refresh button
      notificationRulesGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshNotificationRulesGrid( me.connectionId );
          } );

      notificationRulesGrid.on( 'selectionchange', function ( selectionModel, selectedRecords )
      {
        if ( selectedRecords.length == 0 )
        {
          me._onNotificationRuleUnselected( schedulerNotificationsTab );
        }
        else
        {
          me._onNotificationRuleSelected( selectedRecords[0], schedulerNotificationsTab );
        }
      } );

      notificationRulesGrid.getStore().on( 'beforeload', function ( store, operation )
      {
        notificationRulesGrid.down( '#refreshButton' ).disable();
        notificationRulesGrid.down( '#addNotificationRuleButton' ).disable();
        notificationRulesGrid.down( '#deleteNotificationRuleButton' ).disable();
        notificationRulesGrid.down( '#editNotificationRuleButton' ).disable();
        notificationRulesGrid.down( '#cloneNotificationRuleButton' ).disable();

        /*
         * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
         * Therefore we need to force-fire this event by clearing the selection.
         */
        notificationRulesGrid.getSelectionModel().deselectAll();
      } );

      notificationRulesGrid.getStore().on( 'load', function ( notificationRuleStore, records, successful, operation )
      {
        notificationRulesGrid.down( '#refreshButton' ).enable();
        if ( successful )
        {
          notificationRulesGrid.down( '#addNotificationRuleButton' ).enable();
        }
        else
        {
          notificationRuleStore.removeAll();

          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.InitializeSchedulerExecNotificationsTab.errorLoadingNotifRulesMsg' ) );
        }
      } );

      notificationRulesGrid.down( '#addNotificationRuleButton' ).on( 'click', function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule', {
          connectionId : me.connectionId,
          /*
           * Upon success refresh the notifications tab.
           */
          success : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
              connectionId : me.connectionId
            } ).execute();
          }
        } ).execute();
      } );

      notificationRulesGrid.down( '#deleteNotificationRuleButton' ).on( 'click', function ()
      {
        var notificationRule = notificationRulesGrid.getSelectionModel().getSelection()[0];
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.DeleteExecNotificationRule', {
          connectionId : me.connectionId,
          notificationRule : notificationRule,
          /*
           * Upon success refresh the notifications tab.
           */
          success : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
              connectionId : me.connectionId
            } ).execute();
          }
        } ).execute();
      } );

      notificationRulesGrid.down( '#editNotificationRuleButton' ).on( 'click', function ()
      {
        var notificationRule = notificationRulesGrid.getSelectionModel().getSelection()[0];
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule', {
          connectionId : me.connectionId,
          notificationRuleId : notificationRule.get( 'id' ),
          /*
           * Upon success refresh the notifications tab.
           */
          success : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
              connectionId : me.connectionId
            } ).execute();
          }
        } ).execute();
      } );

      notificationRulesGrid.down( '#cloneNotificationRuleButton' ).on( 'click', function ()
      {
        var notificationRule = notificationRulesGrid.getSelectionModel().getSelection()[0];
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule', {
          connectionId : me.connectionId,
          notificationRuleId : notificationRule.get( 'id' ),
          /*
           * Upon success refresh the notifications tab.
           */
          success : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
              connectionId : me.connectionId
            } ).execute();
          }
        } ).execute();
      } );

      // opens a window with messages enqueued by the notification rule
      notificationRulesGrid.down( '#messagesColumn' ).on( 'requestViewMessages', function ( notificationRuleId )
      {
        Ext.create( 'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator', {
          connectionId : me.connectionId,
          creatorId : notificationRuleId,
          creatorType : QDesk.enums.message.MessageCreatorType.QUARTZ_EXEC_NOTIF_RULE
        } ).execute();
      } );
    }
  },


  /**
   * Invoked when a notification rule record in the SchedulerExecNotificationRulesGrid in the SchedulerNotificationsTab
   * is selected.
   * @param notificationRule a QDesk.model.quartz.ExecNotificationRule instance representing the selected
   * notification rule.
   * @param schedulerNotificationsTab the scheduler Notifications tab.
   */
  _onNotificationRuleSelected : function ( notificationRule, schedulerNotificationsTab )
  {
    var me = this;

    var notificationRulesGrid = schedulerNotificationsTab.down( '#execNotificationRulesGrid' );

    notificationRulesGrid.down( '#deleteNotificationRuleButton' ).enable();
    notificationRulesGrid.down( '#editNotificationRuleButton' ).enable();
    notificationRulesGrid.down( '#cloneNotificationRuleButton' ).enable();
  },


  /**
   * Invoked when a notification rule record in the SchedulerExecNotificationRulesGrid in the SchedulerNotificationsTab
   * is unselected.
   * @param schedulerNotificationsTab the scheduler Notifications tab.
   */
  _onNotificationRuleUnselected : function ( schedulerNotificationsTab )
  {
    var me = this;

    var notificationRulesGrid = schedulerNotificationsTab.down( '#execNotificationRulesGrid' );

    notificationRulesGrid.down( '#deleteNotificationRuleButton' ).disable();
    notificationRulesGrid.down( '#editNotificationRuleButton' ).disable();
    notificationRulesGrid.down( '#cloneNotificationRuleButton' ).disable();
  },


  /**
   * Invoked to refresh the data in the scheduler execution notification rules grid.
   * @param connectionId a connection ID.
   */
  _refreshNotificationRulesGrid : function ( connectionId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationRulesGrid', {
      connectionId : connectionId
    } ).execute();
  }
} );
