/*
 * Copyright (c) 2011-2014 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens the message channel profiles window that allows one to add/edit and delete
 * message channel profiles.
 */
Ext.define( 'QDesk.controller.command.settings.EditAppSettings', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.settings.ApplyAppSettingsChanges',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const',
    'QDesk.view.main.settings.AppSettingsWindow'
  ],

  qdesk : {
    /**
     * MixedCollection instance with unsaved changes.
     */
    changes : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      qdesk : {
        changes : Ext.create( 'Ext.util.MixedCollection' )
      }
    } );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.settings.EditAppSettings.maskMsg' ) );

    // deferred so that the mask message gets displayed
    Ext.defer( function ()
    {
      var me = this;

      var window = Ext.widget( 'AppSettingsWindow' );

      me._bindEventHandlers( window );

      window.on( 'afterrender', function ()
      {
        me.unmask();
      } );

      window.show();
      window.toFront();
    }, 100, me );
  },


  /**
   * Bind message channel profiles window event handlers.
   * @param window a message channel profiles window.
   */
  _bindEventHandlers : function ( window )
  {
    var me = this;

    var treePanel = window.down( '#appSettingsTreePanel' );

    var cardPanel = window.down( '#cardPanel' );

    // selection changes
    treePanel.on( 'selectionchange',
        function ( selectionModel, selectedRecords )
        {
          if ( selectedRecords.length > 0 )  // selection change event is also fired after deselection
          {
            var selectedRecord = selectedRecords[0];
            var nodeId = selectedRecord.get( 'id' );

            var formPanel = me._getFormPanelByNodeId( window, nodeId );
            if ( formPanel )
              cardPanel.getLayout().setActiveItem( formPanel );
          }
          else
          {
            me._onUnselected();
          }
        }, me );

    window.on( 'beforerender', function ()
    {
      me._resetFormValues( window );

      /* Keep track of validity changes*/
      me._bindValidityChangeDetectors( window );

      /* Keep track of changes to the settings */
      me._bindChangeDetectors( window );

      /* Sets initial invalidFieldCount flags in the app settings tree nodes */
      me._detectInvalidFields( window );
    }, me );


    window.down( '#okButton' ).on( 'click', function ()
    {
      if ( me.qdesk.changes.getCount() > 0 )
      {
        var changes = me._getChanges();

        Ext.create( 'QDesk.controller.command.settings.ApplyAppSettingsChanges', {
          changes : changes,
          success : function ()
          {
            me.qdesk.changes.clear();
            window.close();
          }
        } ).execute();
      }
      else
      {
        // no changes
        window.close();
      }
    } );


    window.down( '#applyButton' ).on( 'click', function ( button )
    {
      var changes = me._getChanges();
      Ext.create( 'QDesk.controller.command.settings.ApplyAppSettingsChanges', {
        changes : changes,
        success : function ()
        {
          me.qdesk.changes.clear();
          button.setDisabled( true );
        }
      } ).execute();
    } );
  },


  /**
   * Populates the application setting fields from the current application configuration.
   * @param window the application settings window.
   * @private
   */
  _resetFormValues : function ( window )
  {
    var me = this;

    var fields = Ext.ComponentQuery.query( '#cardPanel form field', window );
    Ext.each( fields, function ( field )
    {
      if ( field.qdesk && field.qdesk.ignore )
      {
        // skip form fields marked with qdesk.ignore flag (various helper fields)
      }
      else
      {
        var fieldName = field.getName();
        var fieldValue = QDesk.Config.get( fieldName );
        field.setValue( fieldValue );
      }
    } );

    me.qdesk.changes.clear();
  },


  /**
   * Bind validity change detectors to all form fields in the specified application settings
   * window.
   * @param window the application settings window.
   * @private
   */
  _bindValidityChangeDetectors : function ( window )
  {
    var me = this;

    var treePanel = window.down( '#appSettingsTreePanel' );

    var fields = Ext.ComponentQuery.query( '#cardPanel form field', window );
    Ext.each( fields, function ( field )
    {
      if ( field.qdesk && field.qdesk.ignore )
      {
        // skip form fields marked with qdesk.ignore flag (various helper fields)
      }
      else
      {
        var okButton = window.down( '#okButton' );
        var applyButton = window.down( '#applyButton' );

        field.on( 'validitychange', function ( field, isValid )
        {
          if ( isValid )
          {
            me._onFieldValid( field );
          }
          else
          {
            me._onFieldInvalid( field );
          }

          var rootNode = treePanel.getRootNode();
          var invalidFieldCount = rootNode.get( 'invalidFieldCount' );

          okButton.setDisabled( invalidFieldCount > 0 );
          applyButton.setDisabled( invalidFieldCount > 0 || me.qdesk.changes.getCount() == 0 );
        } );
      }
    } );
  },


  /**
   * Bind change detectors to all form fields in the specified application settings
   * window.
   * @param window the application settings window.
   * @private
   */
  _bindChangeDetectors : function ( window )
  {
    var me = this;

    var treePanel = window.down( '#appSettingsTreePanel' );

    var fields = Ext.ComponentQuery.query( '#cardPanel form field', window );
    Ext.each( fields, function ( field )
    {
      if ( field.qdesk && field.qdesk.ignore )
      {
        // skip form fields marked with qdesk.ignore flag (various helper fields)
      }
      else
      {
        var fieldName = field.getName();
        var okButton = window.down( '#okButton' );
        var applyButton = window.down( '#applyButton' );

        field.on( 'change', function ( field, newValue, oldValue )
        {
          if ( newValue == QDesk.Config.get( fieldName ) )
          {
            // new value is equal to the currently configured value
            me.qdesk.changes.removeAtKey( fieldName );
            //QDesk.Log.debug( 'Reverted field value, name: ' + fieldName + ', value: ' + newValue );
          }
          else
          {
            me.qdesk.changes.add( fieldName, newValue );
            //QDesk.Log.debug( 'Changed field value, name: ' + fieldName + ', value: ' + newValue );
          }

          var fieldValid = field.isValid();
          var rootNode = treePanel.getRootNode();
          var invalidFieldCount = rootNode.get( 'invalidFieldCount' );

          okButton.setDisabled( invalidFieldCount > 0 || !fieldValid );
          applyButton.setDisabled( invalidFieldCount > 0 || me.qdesk.changes.getCount() == 0 || !fieldValid );
        } );
      }
    } );
  },


  /**
   * Iterates through all forms fields and detects all invalid fields and sets the initial invalidFieldCoount
   * flag values.
   * @param window the application settings window.
   * @private
   */
  _detectInvalidFields : function ( window )
  {
    var me = this;

    var treePanel = window.down( '#appSettingsTreePanel' );

    var fields = Ext.ComponentQuery.query( '#cardPanel form field', window );
    Ext.each( fields, function ( field )
    {
      if ( field.qdesk && field.qdesk.ignore )
      {
        // skip form fields marked with qdesk.ignore flag (various helper fields)
      }
      else
      {
        var okButton = window.down( '#okButton' );
        var applyButton = window.down( '#applyButton' );

        /*
         * We need to set the invalid field counters prior to rendering the window.
         */
        if ( !field.isValid() )
        {
          me._onFieldInvalid( field );
        }

        var rootNode = treePanel.getRootNode();
        var invalidFieldCount = rootNode.get( 'invalidFieldCount' );

        okButton.setDisabled( invalidFieldCount > 0 );
        applyButton.setDisabled( invalidFieldCount > 0 || me.qdesk.changes.getCount() == 0 );
      }
    } );
  },


  /**
   * Returns the current configuration changes that have not been applied yet.
   * @return the object containing the changes.
   * @private
   */
  _getChanges : function ()
  {
    var me = this;

    var changes = {};

    me.qdesk.changes.eachKey( function ( key )
    {
      var value = me.qdesk.changes.getByKey( key );
      changes[key] = value;
    } );

    return changes;
  },


  _onFieldValid : function ( field )
  {
    var me = this;
    var node = me._getNodeByField( field );
    while ( node != null )
    {
      node.set( 'invalidFieldCount', node.get( 'invalidFieldCount' ) - 1 );
      node = node.parentNode;
    }
  },


  _onFieldInvalid : function ( field )
  {
    var me = this;
    var node = me._getNodeByField( field );
    while ( node != null )
    {
      node.set( 'invalidFieldCount', node.get( 'invalidFieldCount' ) + 1 );
      node = node.parentNode;
    }
  },


  /**
   * Returns the form panel for the specified node ID.
   * @param window the application settings window.
   * @param nodeId a node ID.
   * @return {Ext.form.Panel} the form panel, or null.
   * @private
   */
  _getFormPanelByNodeId : function ( window, nodeId )
  {
    var result = null;
    Ext.each( Ext.ComponentQuery.query( '#cardPanel form', window ), function ( formPanel )
    {
      if ( formPanel.qdesk.nodeId == nodeId )
      {
        result = formPanel;
        return false;
      }
    } );

    return result;
  },


  /**
   * Returns the application settings tree node for the specified field.
   * @param field a form field.
   * @return {*} the tree node.
   * @private
   */
  _getNodeByField : function ( field )
  {
    var formPanel = field.up( 'form' );
    var treePanel = formPanel.up( 'window' ).down( '#appSettingsTreePanel' );
    return treePanel.getStore().getNodeById( formPanel.qdesk.nodeId );
  }
} );
